
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <errno.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <resolv.h>
#include <arpa/inet.h>

/* I guess this lot here is Linux-specific really... */ 

/* Read from the serial line in fd, with a prior select lasting 
	ts seconds, tu microseconds.
	Put anything read into the rx length rxcount buffer. 
	returns 0 for timeout, non-0 for tried to read something,
	and *nrd is the count of bytes actually read. */ 

int read_to(int fd, long ts, long tu, char *rx, int rxcount, int *nrd)
{
	fd_set rfds;
	struct timeval tv;
	int retval;

	tv.tv_sec = ts;
	tv.tv_usec = tu; 
	FD_ZERO(&rfds); 
	FD_SET(fd, &rfds); 
	retval = select(fd+1, &rfds, NULL, NULL, &tv); 

	/* fprintf(stderr, "%s(%d): select(%ld, %ld) = %d\n", 
		__FILE__, __LINE__, ts, tu, retval);   */ 

	if(retval)
	{
		*nrd = read(fd, rx, rxcount);

		/* fprintf(stderr, 
			"%s(%d): Read count %d/%d (%d) (%c)\n", 
			__FILE__, __LINE__, *nrd, rxcount, rx[0], rx[0]); */

	}
	return(retval);
}

/* Read a line of text from the device.
	similar to fgets(), read until newline or until 
	the itemsmax-1 are read. 0-terminates the returned string.
	returns NULL on errors and the items when ok.
*/

char *readln(int fd, char *items, size_t itemsmax)
{
	char rx[2];
	int j, nr;
	size_t icount;
	long ts, tu; 
	int retval;
	
	j = 0; 
	icount = 0;
	for(;;)
	{
		/* Select, wait for something to be read, then read it */ 
		if(j == 0)
		{
			ts = 1;
			tu = 0; 
		}
		else
		{
			ts = 0;
			tu = 60000; 
		}

		*rx = 0; 
		retval = read_to(fd, ts, tu, rx, 1, &nr);

		/* fprintf(stderr, "%s(%d): Read %c", __FILE__, __LINE__, *rx);   */ 

		if(retval > 0)
		{
			if(icount < itemsmax+1)
			{
				items[icount] = *rx;
				icount++;
			}

			if(icount == itemsmax-1)
			{
				items[icount] = 0;
				return(items);
			}

			if(*rx == '\n')
			{
				items[icount] = 0;
				return(items);
			}
		}
		else
		if(retval == 0)  /* End or time-out */
		{
			break; 
		}
		else
		if(retval < 0)   /* Error */
		{
			return(NULL);
		}
	}

	items[icount] = 0;
	return(items);
}

/* Perform a simple request/reply conversation with a device 
	sitting at some serial port 
*/ 

int conversate(char *comname, char *cmd, char *reply, int replen)
{
	int j, nw, nr, cdc, fd;
	char *rpcat; 
	char rx[2];
	int retval;
	long ts, tu; 
	// struct termios ti;
	// struct timeval tv;

	/* Linux needs only O_RDWR */ 
	fd = open(comname, O_RDWR );
	if(fd < 0)
	{
		snprintf(reply, replen, 
			"Failure to open %s. errno = %d (%s)\n", 
			comname, errno, strerror(errno) );
		return(0);
	}

	// tcgetattr(fd, &ti); 
	// cfmakeraw(&ti); 
	// /* Linux has the speed included in ti.c_cflag */ 
	// ti.c_cflag = B9600 | CS8 | CREAD | CLOCAL; 
	// ti.c_cc[VMIN] = 0;
	// ti.c_cc[VTIME] = 2;
	// tcsetattr(fd, TCSANOW, &ti); 

	cdc = strlen(cmd);

	/* Write, Linux must then read the echoed character in turn. */ 
	/* provided of course, that the device at the other end is echoing */
	/* the thermometer does not do so, and we don't want to eat the 
		beginning of the returned data either. */ 
	for(j = 0; j<cdc; j++)
	{
		nw = write(fd, cmd+j, 1); 

		/* read_to(fd, 0, 100000, rx, 1, &nr);  */ 
			/* 100 milliseconds time-out 
				Linux must read back */ 
	}
	/* fprintf(stderr, "%s(%d): Wrote %s", __FILE__, __LINE__, cmd);  */ 

	rpcat = reply;
	*rpcat = 0; 
	j = 0; 
	for(;;)
	{
		/* Select, wait for something to be read, then read it */ 
		if(j == 0)
		{
			ts = 1;
			tu = 0; 
		}
		else
		{
			ts = 0;
			tu = 60000; 
		}

		*rx = 0; 
		retval = read_to(fd, ts, tu, rx, 1, &nr);

		/* fprintf(stderr, "%s(%d): Read %c", __FILE__, __LINE__, *rx);   */ 

		if(retval)
		{
			*rpcat++ = *rx;
			j++;
			if(j >= replen) break; 
		}
		else
		{
			break; 
		}
	}
	*rpcat = 0; 
	close(fd); 
	/* fprintf(stderr, "%s(%d): Read %s", __FILE__, __LINE__, reply);  */ 
	return(0);
}


int conversatefd(int fd, char *cmd, char *reply, int replen)
{
	int j, nw, nr, cdc;
	char *rpcat; 
	char rx[2];
	int retval;
	long ts, tu; 

	cdc = strlen(cmd);

	/* Write, Linux must then read the echoed character in turn. */ 
	/* provided of course, that the device at the other end is echoing */
	/* the thermometer does not do so, and we don't want to eat the 
		beginning of the returned data either. */ 

	for(j = 0; j<cdc; j++)
	{
		nw = write(fd, cmd+j, 1); 

		/* read_to(fd, 0, 100000, rx, 1, &nr);  */ 
			/* 100 milliseconds time-out 
				Linux must read back */ 
	}
	/* fprintf(stderr, "%s(%d): Wrote %s", __FILE__, __LINE__, cmd);  */ 

	rpcat = reply;
	*rpcat = 0; 
	j = 0; 
	for(;;)
	{
		/* Select, wait for something to be read, then read it */ 
		if(j == 0)
		{
			ts = 1;
			tu = 0; 
		}
		else
		{
			ts = 0;
			tu = 60000; 
		}

		*rx = 0; 
		retval = read_to(fd, ts, tu, rx, 1, &nr);

		/* fprintf(stderr, "%s(%d): Read %c", __FILE__, __LINE__, *rx);   */ 

		if(retval)
		{
			*rpcat++ = *rx;
			j++;
			if(j >= replen) break; 
		}
		else
		{
			break; 
		}
	}
	*rpcat = 0; 

	/* fprintf(stderr, "%s(%d): Read %s", __FILE__, __LINE__, reply);  */ 
	return(0);
}






